@@ -39,10 +39,12 @@
  *				2. use -Y option in Communicate function
  *				3. in ReadNextChar restore serial port settings and clear lock if exiting
  * modified 13-mar-2010 cjb	1. if yReadPause is set use it
+ * modified 28-mar-2010 cjb	1. working on adding more DSP information
+ * modified 27-jul-2010 cjb	1. added -P option to throttle commands sent to the inverter
  *
  */
 
-char     VersionC[6] = "1.6.0";
+char     VersionC[6] = "1.6.5";
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -69,6 +71,9 @@ BOOL bColumns;
 BOOL bGetInvTime;
 BOOL bGetLocTime;
 BOOL bGetDSP;
+BOOL bGetDSPExtended;
+BOOL bGetDSP3Phase;
+BOOL bHideDSP;
 BOOL bGetEnergy;
 BOOL bCommCheck;
 BOOL bColOutput;
@@ -482,6 +487,147 @@ int GetDSP(int fdser, int yAddress)
     return(0);
 }
 
+void PrintBuffer()
+{
+    int i;
+     for (i = 0; i < cSIZE; i++) {
+        fprintf(stderr, "%02x ",(unsigned char)szSerBuffer[i]);
+    }
+    fprintf(stderr, "\n");
+}
+
+/*--------------------------------------------------------------------------
+    GetDSPExtended
+----------------------------------------------------------------------------*/
+int GetDSPExtended(int fdser, int yAddress)
+{
+    float VB,ILD,ILI,P1,P2,GV,GF,IR,VBD,AGV,VBM,PP,PPT,GVn,WGF,VBp,VBm,ST,AT,HT,T1,T2,T3,F1,F2,F3,F4,F5,PSL,RRB,VPm;
+    char note[3];
+
+    if((VB = GetDSPdata(fdser,yAddress,opGetDSP,ToM5)) < 0) return(-1);
+    if((ILD = GetDSPdata(fdser,yAddress,opGetDSP,ToM6)) < 0) return(-1);
+    if((ILI = GetDSPdata(fdser,yAddress,opGetDSP,ToM7)) < 0) return(-1);
+    if((P1 = GetDSPdata(fdser,yAddress,opGetDSP,ToM8)) < 0) return(-1);
+    if((P2 = GetDSPdata(fdser,yAddress,opGetDSP,ToM9)) < 0) return(-1);
+    if((GV = GetDSPdata(fdser,yAddress,opGetDSP,ToM28)) < 0) return(-1);
+    if((GF = GetDSPdata(fdser,yAddress,opGetDSP,ToM29)) < 0) return(-1);
+    if((IR = GetDSPdata(fdser,yAddress,opGetDSP,ToM30)) < 0) return(-1);
+    if((VBD = GetDSPdata(fdser,yAddress,opGetDSP,ToM31)) < 0) return(-1);
+    if((AGV = GetDSPdata(fdser,yAddress,opGetDSP,ToM32)) < 0) return(-1);
+    if((VBM = GetDSPdata(fdser,yAddress,opGetDSP,ToM33)) < 0) return(-1);
+    if((PP = GetDSPdata(fdser,yAddress,opGetDSP,ToM34)) < 0) return(-1);
+    if((PPT = GetDSPdata(fdser,yAddress,opGetDSP,ToM35)) < 0) return(-1);
+    if((GVn = GetDSPdata(fdser,yAddress,opGetDSP,ToM36)) < 0) return(-1);
+    if((WGF = GetDSPdata(fdser,yAddress,opGetDSP,ToM37)) < 0) return(-1);
+    if((VBp = GetDSPdata(fdser,yAddress,opGetDSP,ToM45)) < 0) return(-1);
+    if((VBm = GetDSPdata(fdser,yAddress,opGetDSP,ToM46)) < 0) return(-1);
+    if((ST = GetDSPdata(fdser,yAddress,opGetDSP,ToM47)) < 0) return(-1);
+    if((AT = GetDSPdata(fdser,yAddress,opGetDSP,ToM48)) < 0) return(-1);
+    if((HT = GetDSPdata(fdser,yAddress,opGetDSP,ToM49)) < 0) return(-1);
+    if((T1 = GetDSPdata(fdser,yAddress,opGetDSP,ToM50)) < 0) return(-1);
+    if((T2 = GetDSPdata(fdser,yAddress,opGetDSP,ToM51)) < 0) return(-1);
+    if((T3 = GetDSPdata(fdser,yAddress,opGetDSP,ToM52)) < 0) return(-1);
+    if((F1 = GetDSPdata(fdser,yAddress,opGetDSP,ToM52)) < 0) return(-1);
+    if((F2 = GetDSPdata(fdser,yAddress,opGetDSP,ToM54)) < 0) return(-1);
+    if((F3 = GetDSPdata(fdser,yAddress,opGetDSP,ToM55)) < 0) return(-1);
+    if((F4 = GetDSPdata(fdser,yAddress,opGetDSP,ToM56)) < 0) return(-1);
+    if((F5 = GetDSPdata(fdser,yAddress,opGetDSP,ToM57)) < 0) return(-1);
+    if((PSL = GetDSPdata(fdser,yAddress,opGetDSP,ToM58)) < 0) return(-1);
+    if((RRB = GetDSPdata(fdser,yAddress,opGetDSP,ToM59)) < 0) return(-1);
+    if((VPm = GetDSPdata(fdser,yAddress,opGetDSP,ToM60)) < 0) return(-1);
+
+    note[0] = '\0';
+    if(bColumns) {
+        if (AGV == VBD) strcpy(note,"*\0");
+        fprintf(outfp, "%11.6f  %11.6f%s  %11.6f%s  %11.6f%s  %11.6f  %11.6f  %11.6f  %11.6f  %11.6f  %11.6f%s  %11.6f%s  %11.6f  %11.6f%s  %11.6f%s  %11.6f%s  %11.6f%s  %11.6f%s  %11.6f%s  %11.6f%s  %11.6f%s  %11.6f%s  %11.6f%s  %11.6f%s  %11.6f%s  %11.6f%s  %11.6f  %11.6f  %11.6f%s  %11.6f%s  %11.6f%s  %11.6f%s  ",VB,VBM,note,VBp,note,VBm,note,VBD,ILD,ILI,IR,GV,AGV,note,GVn,note,GF,PP,note,PPT,note,ST,note,AT,note,HT,note,T1,note,T2,note,T3,note,F1,note,F2,note,F3,note,F4,note,F5,note,P1,P2,PSL,note,RRB,note,VPm,note,WGF,note);
+        bColOutput = TRUE;
+    }
+    else
+    {
+        if (AGV == VBD) strcpy(note," *\0");
+        fprintf(outfp, "\nExtended DSP Reporting\n");
+        fprintf(outfp, "%-31s = %11.6f V\n",_ToM5,VB);
+        fprintf(outfp, "%-31s = %11.6f V  %s\n",_ToM33,VBM,note);
+        fprintf(outfp, "%-31s = %11.6f V  %s\n",_ToM45,VBp,note);
+        fprintf(outfp, "%-31s = %11.6f V  %s\n",_ToM46,VBm,note);
+        fprintf(outfp, "%-31s = %11.6f V\n",_ToM31,VBD);
+        fprintf(outfp, "\n%-31s = %11.6f A\n",_ToM6,ILD);
+        fprintf(outfp, "%-31s = %11.6f A\n",_ToM7,ILI);
+        fprintf(outfp, "%-31s = %11.6f Mohm\n",_ToM30,IR);
+        fprintf(outfp, "\n%-31s = %11.6f V\n",_ToM28,GV);
+        fprintf(outfp, "%-31s = %11.6f V  %s\n",_ToM32,AGV,note);
+        fprintf(outfp, "%-31s = %11.6f V  %s\n",_ToM36,GVn,note);
+        fprintf(outfp, "%-31s = %11.6f Hz\n",_ToM29,GF);
+        fprintf(outfp, "\n%-31s = %11.6f W  %s\n",_ToM34,PP,note);
+        fprintf(outfp, "%-31s = %11.6f W  %s\n",_ToM35,PPT,note);
+        fprintf(outfp, "\n%-31s = %11.6f C  %s\n",_ToM47,ST,note);
+        fprintf(outfp, "%-31s = %11.6f C  %s\n",_ToM48,AT,note);
+        fprintf(outfp, "%-31s = %11.6f C  %s\n",_ToM49,HT,note);
+        fprintf(outfp, "%-31s = %11.6f C  %s\n",_ToM50,T1,note);
+        fprintf(outfp, "%-31s = %11.6f C  %s\n",_ToM51,T2,note);
+        fprintf(outfp, "%-31s = %11.6f C  %s\n",_ToM52,T3,note);
+        fprintf(outfp, "\n%-31s = %11.6f RPM%s\n",_ToM53,F1,note);
+        fprintf(outfp, "%-31s = %11.6f RPM%s\n",_ToM54,F2,note);
+        fprintf(outfp, "%-31s = %11.6f RPM%s\n",_ToM55,F3,note);
+        fprintf(outfp, "%-31s = %11.6f RPM%s\n",_ToM56,F4,note);
+        fprintf(outfp, "%-31s = %11.6f RPM%s\n",_ToM57,F5,note);
+        fprintf(outfp, "\n%-31s = %11.6f W\n",_ToM8,P1);
+        fprintf(outfp, "%-31s = %11.6f W\n",_ToM9,P2);
+        fprintf(outfp, "%-31s = %11.6f W  %s\n",_ToM58,PSL,note);
+        fprintf(outfp, "%-31s = %11.6f V  %s\n",_ToM59,RRB,note);
+        fprintf(outfp, "%-31s = %11.6f V  %s\n",_ToM60,VPm,note);
+        fprintf(outfp, "%-31s = %11.6f Hz %s\n",_ToM37,WGF,note);
+        if (AGV == VBD) fprintf(outfp, "(Note: * = May not be in this Inverter's firmware)\n");
+    }
+
+    return(0);
+}
+
+/*--------------------------------------------------------------------------
+    GetDSP3Phase
+----------------------------------------------------------------------------*/
+int GetDSP3Phase(int fdser, int yAddress)
+{
+    float VBD,GVPn,GVPr,GVPs,GVPt,GCPr,GCPs,GCPt,FRQPr,FRQPs,FRQPt;
+    char note[3];
+
+    if((VBD = GetDSPdata(fdser,yAddress,opGetDSP,ToM31)) < 0) return(-1);
+    if((GVPn = GetDSPdata(fdser,yAddress,opGetDSP,ToM38)) < 0) return(-1);
+    if((GVPr = GetDSPdata(fdser,yAddress,opGetDSP,ToM61)) < 0) return(-1);
+    if((GVPs = GetDSPdata(fdser,yAddress,opGetDSP,ToM62)) < 0) return(-1);
+    if((GVPt = GetDSPdata(fdser,yAddress,opGetDSP,ToM63)) < 0) return(-1);
+    if((GCPr = GetDSPdata(fdser,yAddress,opGetDSP,ToM39)) < 0) return(-1);
+    if((GCPs = GetDSPdata(fdser,yAddress,opGetDSP,ToM40)) < 0) return(-1);
+    if((GCPt = GetDSPdata(fdser,yAddress,opGetDSP,ToM41)) < 0) return(-1);
+    if((FRQPr = GetDSPdata(fdser,yAddress,opGetDSP,ToM42)) < 0) return(-1);
+    if((FRQPs = GetDSPdata(fdser,yAddress,opGetDSP,ToM43)) < 0) return(-1);
+    if((FRQPt = GetDSPdata(fdser,yAddress,opGetDSP,ToM44)) < 0) return(-1);
+
+    note[0] = '\0';
+    if(bColumns) {
+        if (GVPn == VBD) strcpy(note,"*\0");
+        fprintf(outfp, "%11.6f%s  %11.6f%s  %11.6f%s  %11.6f%s  %11.6f%s  %11.6f%s  %11.6f%s  %11.6f%s  %11.6f%s  %11.6f%s  ",GVPn,note,GVPr,note,GVPs,note,GVPt,note,GCPr,note,GCPs,note,GCPt,note,FRQPr,note,FRQPs,note,FRQPt,note);
+        bColOutput = TRUE;
+    }
+    else
+    {
+        if (GVPn == VBD) strcpy(note," *\0");
+        fprintf(outfp, "\n3-Phase DSP Reporting\n");
+        fprintf(outfp, "%-26s = %11.6f V %s\n",_ToM38,GVPn,note);
+        fprintf(outfp, "%-26s = %11.6f V %s\n",_ToM61,GVPr,note);
+        fprintf(outfp, "%-26s = %11.6f V %s\n",_ToM62,GVPs,note);
+        fprintf(outfp, "%-26s = %11.6f V %s\n",_ToM63,GVPt,note);
+        fprintf(outfp, "\n%-26s = %11.6f A %s\n",_ToM39,GCPr,note);
+        fprintf(outfp, "%-26s = %11.6f A %s\n",_ToM40,GCPs,note);
+        fprintf(outfp, "%-26s = %11.6f A %s\n",_ToM41,GCPt,note);
+        fprintf(outfp, "\n%-26s = %11.6f Hz%s\n",_ToM42,FRQPr,note);
+        fprintf(outfp, "%-26s = %11.6f Hz%s\n",_ToM43,FRQPs,note);
+        fprintf(outfp, "%-26s = %11.6f Hz%s\n",_ToM44,FRQPt,note);
+        if (GVPn == VBD) fprintf(outfp, "(Note: * = May not be in this Inverter's firmware)\n");
+    }
+
+    return(0);
+}
 
 /*--------------------------------------------------------------------------
     GetDSPdata
@@ -498,6 +644,7 @@ float GetDSPdata(int fdser, int yAddress, int opcode, int param)
     szSerBuffer[cCommandEnd+1] = '\0';
     nCnt = Communicate(fdser, yAddress);
     if (nCnt > 0 && TransState((int)szSerBuffer[aState])) {
+
         paramValFloat = cvrtFloat(szSerBuffer);
         if(bVerbose) fprintf(stderr, "value     %12.6f\n",*paramValFloat);
     }
@@ -718,12 +865,28 @@ int Communicate(int fdser, int yAddress)
     char szSerBufferSave[_szSerBufferLen];
     int CRCrc = -1;
     int attempts = 1;
+    struct timeval curtv;
+    long long int curtvusecs = 0, lastcommtvusecs = 0, elapsedtvusecs = 0;
 
     strcpy(szSerBufferSave, _clearCMD);
     strcpy(szSerBufferSave, szSerBuffer);
+    gettimeofday(&curtv, NULL);
     while(CRCrc < 0 && attempts <= yMaxAttempts) {
+        if(lastcommtv.tv_sec == 0 || lastcommtv.tv_usec == 0) {
+            lastcommtv.tv_sec = curtv.tv_sec;
+            lastcommtv.tv_usec = curtv.tv_usec;
+        } else {
+            curtvusecs = (curtv.tv_sec*1000000) + curtv.tv_usec;
+            lastcommtvusecs = (lastcommtv.tv_sec*1000000) + lastcommtv.tv_usec;
+            elapsedtvusecs = curtvusecs - lastcommtvusecs;
+        }
         strcpy(szSerBuffer, szSerBufferSave);
-        if(bVerbose) fprintf(stderr, "\nAttempt %i\nClearing read buffer  ",attempts);
+        if(bVerbose) fprintf(stderr, "\nElapsed time since last comm %llu uS\nAttempt %i",elapsedtvusecs,attempts);
+        if(yCommPause > 0 && elapsedtvusecs < yCommPause) {
+            if(bVerbose) fprintf(stderr, " Sleeping for %llu uS",yCommPause-elapsedtvusecs);
+            usleep(yCommPause-elapsedtvusecs);
+        }
+        if(bVerbose) fprintf(stderr, "\nClearing read buffer  ");
         while((rc = ReadNextChar(fdser, &ch, 0)) && i < _clrAttemps) {
             if(bVerbose) fprintf(stderr, ":ch=%i ",ch);		/* clear channel and delay */
             i++;
@@ -785,6 +948,7 @@ int Communicate(int fdser, int yAddress)
                 if (bRptReadPause) fprintf(stderr, "\n%s: %s: Waiting %d milli-seconds before reading inverter response",RunTime,ProgramName,yReadPause);
             usleep(yReadPause*1000);
         }
+        tcflush(fdser,TCIFLUSH);
         nCnt = ReadToBuffer(fdser, szSerBuffer, aSIZE);
         if(bVerbose) {
             fprintf(stderr, "answer:  ");
@@ -809,7 +973,10 @@ int Communicate(int fdser, int yAddress)
                 CRCrc = 0;
             }
         }
-    attempts++;
+        gettimeofday(&curtv, NULL);
+        lastcommtv.tv_sec = curtv.tv_sec;
+        lastcommtv.tv_usec = curtv.tv_usec;
+        attempts++;
     }
     if (CRCrc < 0) return(-1);
     if (bRptReties) {
