@@ -27,7 +27,7 @@
  *
  */
 
-#define	VersionCH	"1.6.0"
+#define	VersionCH	"1.6.5"
 
 extern int CommCheck(int fdser, int yAddress);
 extern int GetState(int fdser, int yAddress);
@@ -38,6 +38,8 @@ extern int GetVerFW(int fdser, int yAddress);
 extern int GetMfgDate(int fdser, int yAddress);
 extern int GetConf(int fdser, int yAddress);
 extern int GetDSP(int fdser, int yAddress);
+extern int GetDSPExtended(int fdser, int yAddress);
+extern int GetDSP3Phase(int fdser, int yAddress);
 extern int GetCE(int fdser, int yAddress);
 extern int GetJoules(int fdser, int yAddress);
 extern int GetCounters(int fdser, int yAddress);
