@@ -42,10 +42,12 @@
  * modified 13-mar-2010 cjb	1. added -P option to delay read after sending command to inverter
  *				2. rename -P to -U and add -u to report it
  *				3. changed -U seconds to wait to milli-seconds to wait
+ * modified 28-mar-2010 cjb	1. working on adding more DSP information
+ * modified 27-jul-2010 cjb	1. added -P option to throttle commands sent to the inverter
  *
  */
 
-static char	VersionM[] = "1.6.0";
+static char	VersionM[] = "1.6.6";
 char     VersionC[6];
 
 #include <syscall.h>
@@ -72,6 +74,9 @@ char     VersionC[6];
 BOOL bVerbose = FALSE;
 BOOL bColumns = FALSE;			/* Output data in columns */
 BOOL bGetDSP = FALSE;			/* Measure request to the DSP */
+BOOL bGetDSPEntedned = FALSE;		/* Measure request to the DSP more parameters */
+BOOL bGetDSP3Phase = FALSE;		/* Measure request to the DSP three-phase parameter */
+BOOL bHideDSP = FALSE;
 BOOL bGetEnergy = FALSE;		/* Cumulated Energy Readings */
 BOOL bGetInvTime = FALSE;		/* Display Inverter time flag */
 BOOL bGetLocTime = FALSE;		/* Display computer time flag */
@@ -85,6 +90,8 @@ int yTimeout = 0;			/* read timeout value in uS */
 long PID;
 int yMaxAttempts = 1;
 int yReadPause = 0;
+long int yCommPause = 0;
+struct timeval lastcommtv;
 
 FILE *outfp;
 
@@ -142,6 +149,8 @@ int main(int argc, char *argv[])
     char command[255];
     char *SubStrPos = NULL;
 
+    lastcommtv.tv_sec = 0;
+    lastcommtv.tv_usec = 0;
     timeValue = time(NULL);
     timStruct = *(localtime(&timeValue));
     strftime(RunTime,18,"%Y%m%d-%H:%M:%S ",&timStruct);
@@ -155,10 +164,13 @@ int main(int argc, char *argv[])
         printf("Options:\n");
         printf(" -b, --verbose                  Verbose mode\n");
         printf(" -C, --calc-value=num           Calculate $ value using num * KWh\n");
-        printf(" -c, --columnize                Output data in columns --> for -d & -e options\n");
-        printf("                                  only, will disable all other options\n");
+        printf(" -c, --columnize                Output data in columns --> for -d, -e, -D, & -E options only\n");
+        printf("                                  will disable all other options -- if value ends with an \"*\"\n");
+        printf("                                  reporting of that item may not be in inverters firmware\n");
         printf(" -d, --get-dsp                  Get DSP data\n");
+        printf(" -D, --get-dsp-extended         Get more DSP data\n");
         printf(" -e, --get-energy               Get Cumulated Energy readings\n");
+        printf(" -E, --get-dsp-3phase           Get 3-Phase DSP data\n");
         printf(" -f, --firmware-ver             Query for Firmware Version string\n");
         printf(" -g, --mfg-date                 Query for Inverter Manufacturing Date\n");
         printf(" -h, --help                     This text\n");
@@ -169,6 +181,7 @@ int main(int argc, char *argv[])
         printf(" -n, --serial-number            Query for Inverter Serial Number\n");
         printf(" -o, --output-file=<filename>   Append data to file (Created if nonexistant)\n");
         printf(" -p, --part-number              Query for Inverter Part Number\n");
+        printf(" -P, --comm-pause=num           Wait <num> uS between sending commands to inverter (1-1000000)\n");
 	printf(" -R, --read-timeout=num		Timeout value when reading data from the Inverter (mS)\n");
         printf(" -r, --calc-grid-power          Calc Grid power using Grid Voltage * Grid Current,\n");
         printf("                                  instead of reporting the Inverter's value. --> for \n");
@@ -385,12 +398,6 @@ int main(int argc, char *argv[])
         if (rc == 0 && bGetTime)
             rc |= GetTime(fdser,yAddress);
 
-        if (rc == 0 && bGetDSP)
-            rc |= GetDSP(fdser,yAddress);
-
-        if (rc == 0 && bGetEnergy)
-            rc |= GetCE(fdser,yAddress);
-
         if (!bColumns) {
 
             if (rc == 0 && bGetPN)
@@ -420,6 +427,20 @@ int main(int argc, char *argv[])
             if (rc == 0 && bGetCount)
                 rc |= GetCounters(fdser,yAddress);
         }
+
+        if (rc == 0 && bGetDSP)
+            rc |= GetDSP(fdser,yAddress);
+
+        if (rc == 0 && bGetEnergy)
+            rc |= GetCE(fdser,yAddress);
+
+        if (rc == 0 && bGetDSPExtended)
+            rc |= GetDSPExtended(fdser,yAddress);
+
+
+        if (rc == 0 && bGetDSP3Phase)
+            rc |= GetDSP3Phase(fdser,yAddress);
+
         if(bVerbose) fprintf(stderr, "\nrc: %d\n",rc);
     }
     if(rc >= 0) {
@@ -558,9 +579,12 @@ int GetParms(int argc, char *argv[])
 	{ "calc-value",		required_argument,	0,	'C' },
         { "columnize",		no_argument,		0,	'c' },
         { "get-dsp",		no_argument,		0,	'd' },
+        { "get-dsp-extended",	no_argument,		0,	'D' },
+        { "get-dsp-3phase",     no_argument,		0,	'E' },
         { "get-energy",		no_argument,		0,	'e' },
         { "firmware-ver",	no_argument,		0,	'f' },
         { "mfg-date",		no_argument,		0,	'g' },
+        { "hide-dsp-msg",	no_argument,		0,	'H' },
         { "help",		no_argument,		0,	'h' },
         { "get-count",		no_argument,		0,	'i' },
         { "get-joules",		no_argument,		0,	'j' },
@@ -569,6 +593,7 @@ int GetParms(int argc, char *argv[])
         { "get-conf",		no_argument,		0,      'm' },
         { "serial-number",	no_argument,		0,	'n' },
         { "part-number",	no_argument,		0,	'p' },
+        { "comm-pause",		required_argument,	0,	'P' },
         { "read-timeout",	required_argument,	0,	'R' },
         { "calc-grid-power",	no_argument,		0,	'r' },
         { "set-time",		no_argument,		0,	'S' },
@@ -594,7 +619,7 @@ int GetParms(int argc, char *argv[])
     if(argc == 1)
         return 0;				/* no parms at all */
 
-    while ((c = getopt_long(argc, argv, "a:bC:cdefghijl:mno:pR:rSsTtVvU:uw:xY:y", longopts, NULL )) != EOF) {
+    while ((c = getopt_long(argc, argv, "a:bC:cDdEefgHhijl:mno:P:pR:rSsTtVvU:uw:xY:y", longopts, NULL )) != EOF) {
         switch (c) {
             case 'a':
                 /* Inverter address */
@@ -606,9 +631,12 @@ int GetParms(int argc, char *argv[])
                 break;
             case 'c': bColumns     = TRUE; break;
             case 'd': bGetDSP      = TRUE; break;
+            case 'D': bGetDSPExtended    = TRUE; break;
+            case 'E': bGetDSP3Phase      = TRUE; break;
             case 'e': bGetEnergy   = TRUE; break;
             case 'f': bGetVerFW    = TRUE; break;
             case 'g': bGetMfg      = TRUE; break;
+            case 'H': bHideDSP     = TRUE; break;
             case 'h': bHelp        = TRUE; break;
             case 'i': bGetCount    = TRUE; break;
             case 'j': bGetJoules   = TRUE; break;
@@ -625,6 +653,13 @@ int GetParms(int argc, char *argv[])
             case 'n': bGetSN       = TRUE; break;
             case 'o': strcpy(outfile, optarg); break;
             case 'p': bGetPN       = TRUE; break;
+            case 'P':
+                yCommPause = atoi(optarg);
+                if (yCommPause <= 0 || yCommPause > 1000000) {
+                    fprintf(stderr, "\n%s: %s: Comm Pause micro-seconds (%li) out of range, 1-1000000.\n",RunTime,ProgramName,yCommPause);
+                    return 0;
+                }
+                break;
             case 'R':
                 /* read timeout value in uS */
                 yTimeout = atoi(optarg)*1000;
